 /*
Routine to read JEM-X data from fits file and generate shadowgram.

Aug. 2003 Modified by Niels Lund to extract data from fits file
Dec. 2003 version cleaned up wrt output and reading of fits files.
          Note: does not support restricted image format!! (yet!)
*/

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<string.h>

#define listlength 220000
#define firstevent 32000.0

/*#include<dos.h>*/

double calsrcx[2][4] = {38.25, 103.25, 152.25, 216.75, 38.25, 103.25, 152.25, 216.75}; 
double calsrcy[2][4] = {197.5, 235.0, 234.25, 197.0, 197.0, 235.0, 234.5, 197.0}; 
double calrad[2][4] = {7.5, 5.0, 6.5, 5.0, 5.0, 5.0, 5.0, 5.0};

/* 	!!!!!!!! NEW ph-thresholds for definition of energy bands !!!!!!!!!!!!!!!!!!!!!! 
              if ((ri_pha >= 1) && (ri_pha < 3) ee = 0;	
              if ((ri_pha >= 3) && (ri_pha < 5) ee = 1;
              if (ri_pha == 5) ee = 2;
              if (ri_pha >= 6) ee = 3;
        MAY 2004:      
        !!!!!!! The "ri-anchoring" of the energy channels is only valid at one particular epoch
        !!!!!!! I use orbit 102 as the reference, and try to scale the channel definitions for other epochs.
        !!!!!!! (expressed in a 1024 channel linear scale)

   *     SB values for the Cal. source 22 keV peak positions during Crab calibration runs (4096 ch scale):
   *     (using Carol-Annes conversions from cal. source channel to Xe-peak channel
   *       orbit    JEM-X1   use:  Xe-ch        JEM-X2           use:  Xe-ch
   *       42-45    307,317  312    425       478,464,502,582    464    645
   *        102     340,346  343    468       521,476,520,622    476    662
   *        170     385,393  389    530       615,546,603,726    546    760
              
                      ri_pha:   0      1      2      3      4      5      6      7
       corresp. 4096 channel:   0     32     64     96    128    192    320    576   1040 (on-board upper lim: 1200)
       corresp.  nl  channel:   0      8     16     24     32     48     80    144    260 
 crude keV (30 keV @ ch 662):   0    1.4    2.9    4.4    5.8    8.7    14.5  26.1   47.1
      absorbtion length (mm):      -      0.0    1.0    2.0    4.3   10.0  40.0   70.0
  absorbtion length (e-band):                1.0 mm       3.5 mm     10.0mm |     45.0 mm
X-ray raw count distribution:     0.05%   2.3%  16.4%  21.0%  26.3%  18.3%  | 8.1%   7.6%            (orbit 108)
                 energy band:           |    0000     |    1111     | 2222  |       3333        |    
     estmated position sigma:  inf          2.1 mm 1.6 mm 1.2 mm 0.8 mm .46 mm .46         0.6 mm

Matching selection boundaries expressed in "nl-channels" for three Crab calibration epochs:

   rev         JEM-X1                                   JEM-X2
              Xe-peak   Boundaries                    Xe-peak         Boundaries
  42-45         425     5   15   31   51   167          645           8   23   47   78   253
   102          468     6   17   34   57   184          662 (ref)     8   24   48   80   260
   170          530     6   19   38   64   208          760           9   28   55   92   298

*/

  /* pulseheight selection boundaries */
int pha_sel[3][2][5] = { 
   5, 15, 31, 51, 167, 8, 23, 47, 78, 253,
   6, 17, 34, 57, 184, 8, 24, 48, 80, 260,
   6, 19, 38, 64, 208, 8, 28, 55, 92, 298 };
/* these boundaries corresponds roughly to 2, 4.2, 8.4, 14 and 35 keV (with reduced voltage setting) */ 
   
/*
int pha_sel[3][2][5] = { */	/* special selection providing two energy channels above 14 keV */ 
/*   5, 15, 51,  92, 167, 8, 23, 78, 140, 253,
   6, 17, 57, 102, 184, 8, 24, 80, 144, 260,
   6, 19, 64, 114, 208, 8, 28, 92, 164, 298 }; */
/* these boundaries corresponds roughly to 2, 4.2, 14, 26 and 35 keV (with reduced voltage setting) */ 


int sel0, sel1, sel2, sel3, sel4;

short shdgr[4][256][256];
int limit[4][101], m_max;
double snratio[4][101];
char sweethome[200];
unsigned char fitsdat[10000000];

float pha_xy(int x, int y, int pha, int jemx, char iname2[], int print);

/* ***************************************************************************** */

main(argc,argv)
int argc ;
char *argv[] ;
{
	FILE *in, *fim, *sm_fil;
	FILE *spgnu ;
	FILE *spec ;
	FILE *stat ;
	FILE *imagegnu ;
	FILE *cntsgnu ;
	FILE *hkgnu ;        
        FILE *sp_fil;
        FILE *cfil;
        FILE *cnt_file;

        char xtension[30], *item_st, *item_end;
        unsigned char *x_ptr, *tab_ptr, *nax1, *nax2, *dataptr, *end_ptr, *listptr;
	char iname[100], oname[100], dummy[1024], statname[128], b_file[100], E_name[32];
	char iname2[100], spec_file[100], im_file[100], hk_file[100], filnavn[100]; 
        unsigned char cntgrey[listlength]; 
        unsigned char cntratel[listlength], cntratem[listlength], cntrateh[listlength];
        
	unsigned short inb[16], buff_loss[listlength], hkssc[listlength], et_time[listlength];
	unsigned short tm[220], mid, ssc, cal_amp, cal_n, tmdum[220];
        unsigned short hkdat[20000][60], hkcur[30], a_counts[256][2000], anode_sum[256];
	short grey, cur_grey = -1;
        unsigned short fits_sh;

        int spect[256], pspect[256], pack_stat[256], sp[256], dsp[256], rspect[256];
        int xfn[256], yfn[256], xyfn[16][16], xcspec[256], ycspec[256], xycspec[256];
        int rawspec[256], selspect[256], seltotsp[256], p_fold[60][64], sev0, sev1, sev2, sev3;
        int calspec[4][256], waittim[256];
	int outblock[35], psum, first=1, hkstart, status, calseg, spin, last_nev, last_pos;
	int i, j, k, n, m, apid, apidold=-1, nsame = 0, ev_amp, an_stat, nev, ppha, ptime_i;
	int offset, pos, hkold, hknum, hkwrap, pid, pidold, ii, jj, pickevent, esc;
        int pidevents, pidtime, b_open, jemx=0, tmnum=0, xfsum, pidselect, lolim, hilim, ipack;
        int cntindex = -1, start_ssc = -1, ire_time, ilimit, stop_ssc, old_ssc, nprint=0;
        int a_cnt_index=0, dayshift, dayfix=0, sci_lobt1, num_tot[4], map_open, pidnext;
        int OnTargetFlag, P_duration, oevnum, item_len;
        int pha, x, y, dti, divid, pidv ;
        int filelen, tab_wdth1, tab_wdth2, tab_wdth3, tab_dpth1, tab_dpth2, tab_dpth3;
        int it1, it2, it3, ssc_old, ssc_grey, pos_grey, period, iipha;

        unsigned short SSC_1[5000], lobt2_1[5000], lobt1_1[5000], lobt0_1[5000];
        unsigned short n_evts[5000], grey_1[5000], n_grey[5000], idum;
        unsigned short SSC_3[200], lobt2_2[200], lobt1_2[200], lobt0_2[200], pos_gr[200], grey_3[200], delT_gr[200];
        unsigned rowbg_1[5000];


	unsigned short anode ;
        unsigned char cpha, cx, cy, cgrey; 
	unsigned int lobt[4] ;  /* we have to use 4 byte ints as idl has no unsigned */
        int nblok[256], nevent[256];
	int n1602 = 0, n1536 = 0, n1542 = 0 ;
	int n1608 = 0, nev1608 = 0 ;
        
        float ccpha, e_rate;
        
	double packtime, packtime0, packrel, delta, dt, xcsum, raver, e_time, m0, m1, m2, aver, var;
        double xfitin[256], yfitin[256], xyfitin[16][16], cntratm, gc, hk_time, acc_curT;
        double xcor[256], ycor[256], xycor[16][16], hk_curt[listlength], hk_curtime, re_time, k_fact, acTlim;
        double cyclow, cychigh, imageT0 = 0.0, imageT1 = 0.0, accumT, cyclownext, BCPtime, full_evtime;
        double hknorm[60] =
           {1.0, 1.0, 1.0, 1.0, 1.0, 8.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0, 8.0, 8.0, 8.0,
            8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
            8.0, 8.0, 8.0, 8.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0, 8.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0, 8.0, 8.0, 8.0,
            8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
            8.0, 8.0, 8.0, 8.0, 1.0, 1.0};
        double month_L[12] = {1,  32,  60,  91, 121, 152, 182, 213, 244, 305, 335, 366};
        
        double UToffset, hk_tid, UT_time, hourofyear, hk_tider[20000], UT_res0[2], livetime = 0.0;
        
        double xrdet, yrdet, ydet2, rmean[26], normx, norm0, aux_tot[30], rrdet, startT, runT;
        
	int dayofyear, science=0, radm0[26], radm1[26], radm2[26], irdet;
        int pspsuma, pspsumb, st_lobt1, value, orbit, nn;


/* restricted imaging count rate data */
	int n1656 = 0, nev1656 = 0 ;
	char cratebuf[406] ;
	unsigned short count, mid1, mid2 ;

/* restricted imaging */
	unsigned int n1624 = 0, nev1624 = 0, imseq, nimg1624=0, pha8 ;
	unsigned short xy, deltaxy, xbuf[5000], ybuf[5000], phabuf[5000] ;
        char iline[300] ;
	char phline[128] ;

        /* ********************************************* */
        
        UToffset = 6960.0 + 5.66 - 4.34 + 0.38;	/* (hours since Jan 1st 2002 00:00:00) */ 

        sprintf ( sweethome, "/home/nl/jemx/c_soft" ) ;

        sprintf(xtension, "XTENSION= 'BINTABLE'");

        /* for 2003: 2003 Jan. 12th, UT 14:30:30 = 7563135 in BCPtime */
        
        for (i=0; i<256; i++) rspect[i] = pspect[i] = spect[i] = pack_stat[i] = selspect[i] = 0;
        for (i=0; i<256; i++) calspec[0][i] = calspec[1][i] = calspec[2][i] = calspec[3][i] = waittim[i] = 0;
        for (i=0; i<256; i++) xcspec[i] = ycspec[i] = xycspec[i] = rawspec[i] = seltotsp[i] = 0;
        for (i=0; i<64; i++) for (k=0; k<60; k++) p_fold[k][i] = 0;
        for (i=0; i<256; i++) for (j=0; j<2000; j++) a_counts[i][j] = 0;
        for (i=0; i<26; i++) radm0[i] = radm1[i] = radm2[i] = 0;
        
        for (i=0; i<16; i++) 
           for (j=0; j<16; j++) {xyfitin[i][j] = 0.0; xycor[i][j] = 1.0; xyfn[i][j] = 0;}
        
        for (i=0; i<listlength; i++) {
           cntratel[i] = cntratem[i] = cntrateh[i] = cntgrey[i] = buff_loss[i] = 0;
           hk_curt[i] = 0.0;
        }
                
        hknum = hkold = hkwrap = 0;
        pid = pidtime = pidevents = 0;
        pidold = 999;
        b_open = map_open = 0;
        jemx = 1;
        pidselect = pidnext = -1;
        lolim = 0;
        hilim = 256;
        cyclow = cyclownext = 1000000.0;
        cychigh = 1000000.0;
        dayshift = 0;
        dayfix = 0;
        divid = 1;
        pidv = 0;
        accumT = 1800.0;
        
	if (argc > 1) strcpy(iname,argv[1]) ;
        else {printf("fits-file {accumT(s) divide{1}}\n"); return;}
        if (argc > 2) accumT = atof(argv[2]) ;           
        if (argc > 3) divid = atoi(argv[3]) ;           

        acTlim = accumT / (double)(divid);
        
        strcpy(iname2, iname);
        strcat(iname, ".fits"); 
	in = fopen(iname,"rb") ;
	if ( in == NULL)
	{
	  printf("%s, file not found\n", iname) ;
	  return ;
	}
        
        jemx = -1;

        item_st = strstr(iname, "/");

/*        if (item_st == NULL) printf(" / not found\n");
        else printf(" / found at position: %2d\n", (int)(item_st - iname));
*/        
        if (item_st == NULL) item_end = iname; 
        else { 
moresl:	   item_end = item_st+1;
   	   item_st = strstr(item_end, "/");

/*           if (item_st == NULL) printf(" / not found\n");
           else printf(" / found at position: %2d\n", (int)(item_st - iname));
*/        
           if (item_st != NULL) goto moresl;
        }
        
        item_st = item_end+1;

        item_end = strstr(item_st, "_");
/*        if (item_end == NULL) {printf(" _ not found\n"); goto namex;}
        else printf(" _ found at position: %2d\n", (int)(item_end - iname));
*/        
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        jemx = atoi(iline) - 1;

        item_st = item_end+1;

        item_end = strstr(item_st, "_");
/*        if (item_end == NULL) {printf(" _ not found\n"); goto namex;}
        else printf(" _ found at position: %2d\n", (int)(item_end - iname));
*/        
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        orbit = atoi(iline);
        
        item_st = item_end+1;
        item_end = strstr(item_st, "_");
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        pid = atoi(iline);
        
        item_st = item_end+1;
        item_end = strstr(item_st, ".");
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        pidv = atoi(iline);
        
        if (jemx < 0) {
namex:     printf("Could not interpret fits file name: %s\n", iname);
           return;
        }
        
        printf("%s interpreted as: %1d, %3d, %4d, %3d\n", iname, jemx+1, orbit, pid, pidv);
        
        sprintf(filnavn, "%s/reference/phtabl2.txt", sweethome);
        spec = fopen(filnavn, "rt");
        if (spec == NULL) {
           printf("Error opening file: %s ..... exiting!\n", filnavn) ;
           return ;
        }

        i = k = 0;
        for (i=0; i<256; i+=16) {
           fgets(phline, 128, spec);
           sscanf(phline, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
              &sp[i+0], &sp[i+1], &sp[i+2], &sp[i+3], &sp[i+4], &sp[i+5], &sp[i+6], &sp[i+7],    
              &sp[i+8], &sp[i+9], &sp[i+10], &sp[i+11], &sp[i+12], &sp[i+13], &sp[i+14], &sp[i+15]);
        }
        
        dsp[0] = 1;
        dsp[1] = 2;
        for (i=2; i<256; i++) dsp[i] = sp[i]-sp[i-1];
        
        fclose(spec);
/*        printf("Initialized pha translation table\n");
*/        
        j = 5;
        for (i=0; i<4; i++) {
           aux_tot[j++] = calsrcx[jemx][i];	/* cal source positions & elimination radii */
           aux_tot[j++] = calsrcy[jemx][i];
           aux_tot[j++] = calrad[jemx][i];
        }

        period = 1;
        if (orbit < 71) period = 0;
        if (orbit > 140) period = 2;
        
        sel0 = aux_tot[17] = pha_sel[period][jemx][0];	/* pulseheight selection boundaries */
        sel1 = aux_tot[18] = pha_sel[period][jemx][1];
        sel2 = aux_tot[19] = pha_sel[period][jemx][2];
        sel3 = aux_tot[20] = pha_sel[period][jemx][3];
        sel4 = aux_tot[21] = pha_sel[period][jemx][4];
        printf("pha_sel: %4d %4d %4d %4d %4d\n", sel0, sel1, sel2, sel3, sel4);

        aux_tot[22] = jemx;
   
        jj = 0;

        i = strlen(iname);
           
        filelen = fread(fitsdat, 1, 10000000, in);
/*        printf("Number of bytes read: %10d\n", filelen);
*/           
        if ((x_ptr = strstr(fitsdat, xtension)) == NULL) {printf("null-pointer 00\n"); exit(0);}
/*        printf("dataptr1-fitsdat: %12d, filelen: %12d\n", (int)(x_ptr-fitsdat), filelen);
*/
           
        if ((nax1 = strstr(x_ptr, "NAXIS1")) == NULL) {printf("null-pointer 01\n"); exit(0);}
        *(nax1+31) = 0;
        tab_wdth1 = atoi(nax1+16);
/*        printf("Table width1: %5d\n", tab_wdth1);
*/  
        if ((nax2 = strstr(nax1+32, "NAXIS2")) == NULL) {printf("null-pointer 02\n"); exit(0);}
        *(nax2+31) = 0;
        tab_dpth1 = atoi(nax2+16);
/*        printf("Table depth1: %10d\n", tab_dpth1);
*/           
        nax2 += 32;

        if ((nax2 = strstr(nax2+32, "EXTNAME")) == NULL) {printf("null-pointer 02a\n"); exit(0);}
        strncpy(E_name, nax2+16, 4);
        E_name[4] = 0;
/*        if ((strcmp(E_name, "REST") * strcmp(E_name, "FULL")) != 0) { */
        if (strcmp(E_name, "FULL") != 0) {
           printf("Fits file %s discarded, extension 'name': %s\n", iname, E_name);
           return;
        }
        
        nax2 += 32;

        if ((end_ptr = strstr(nax2, "END    ")) == NULL) {printf("null-pointer 03\n"); exit(0);}
        if ((end_ptr - x_ptr) % 80 != 0) {
           printf("END -marker found out of place\n");
           exit(0);
        }
/*        printf("endptr-fitsdat: %12d\n", (int)(end_ptr-fitsdat));
*/        
        if (((end_ptr - fitsdat) % 2880) == 0) dataptr = end_ptr;   
        else dataptr = fitsdat + ((int)((end_ptr - fitsdat) / 2880) + 1) * 2880;

/*        printf("dataptr-fitsdat: %12d\n", (int)(dataptr-fitsdat));
*/
        if ((dataptr - fitsdat) > filelen) {
           printf("dataptr-fitsdat: %12d, filelen1: %12d\n", (int)(dataptr-fitsdat), filelen);
           return;
        } 
        
        for (i=0; i<tab_dpth1; i++) {
           fits_sh    = *dataptr * 256 + *(dataptr+1);
           SSC_1[i]   = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+4) * 256 + *(dataptr+5);
           lobt2_1[i] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+6) * 256 + *(dataptr+7);
           lobt1_1[i] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+8) * 256 + *(dataptr+9);
           lobt0_1[i] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);              
              
           fits_sh    = *(dataptr+10) * 256 + *(dataptr+11);
           rowbg_1[i] = ((int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000)) << 16;
           rowbg_1[i] += *(dataptr+12) * 256 + *(dataptr+13);

           fits_sh    = *(dataptr+14) * 256 + *(dataptr+15);
           n_evts[i]  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+18) * 256 + *(dataptr+19);
           n_grey[i]  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           grey_1[i]  = *(dataptr+20);

/*           if (i < 20) printf("ssc: %5d, lobt2/1/0: %5d %5d %5d, row_begin: %6d, events: %3d, grey: %2d\n",
              SSC_1[i], lobt2_1[i], lobt1_1[i], lobt0_1[i], rowbg_1[i], n_evts[i], grey_1[i]);
*/
           dataptr += tab_wdth1;
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen2: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        }

/*        printf("endptr-fitsdat: %12d\n", (int)(end_ptr-fitsdat));
*/        
        if (((dataptr - fitsdat) % 2880) != 0)    
           dataptr = fitsdat + ((int)((dataptr - fitsdat) / 2880) + 1) * 2880;

/*        printf("dataptr-fitsdat: %12d, string: %s\n", (int)(dataptr-fitsdat), xtension);
*/
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen3: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        
        if ((x_ptr = strstr(dataptr, xtension)) == NULL) {printf("null-pointer 04\n"); exit(0);}
/*        printf("dataptr2-fitsdat: %12d, filelen: %12d\n", (int)(x_ptr-fitsdat), filelen);
*/           
        if ((nax1 = strstr(x_ptr, "NAXIS1")) == NULL) {printf("null-pointer 05\n"); exit(0);}
        *(nax1+31) = 0;
        tab_wdth2 = atoi(nax1+16);
/*        printf("Table width2: %5d\n", tab_wdth2);
*/        
        if ((nax2 = strstr(nax1+32, "NAXIS2")) == NULL) {printf("null-pointer 06\n"); exit(0);}
        *(nax2+31) = 0;
        tab_dpth2 = atoi(nax2+16);
/*        printf("Table depth2: %10d\n", tab_dpth2);
*/           
        nax2 += 32;
        if ((end_ptr = strstr(nax2, "END    ")) == NULL) {printf("null-pointer 07\n"); exit(0);}
        if ((end_ptr - x_ptr) % 80 != 0) {
           printf("END -marker found out of place\n");
           exit(0);
        }
           
        if (((end_ptr - fitsdat) % 2880) == 0) dataptr = end_ptr;   
        else dataptr = x_ptr + ((int)((end_ptr - x_ptr) / 2880) + 1) * 2880;
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen4: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        listptr = dataptr;
        
        dataptr += tab_wdth2 * tab_dpth2;
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen5: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 

        if (((dataptr - fitsdat) % 2880) != 0)   
           dataptr = fitsdat + ((int)((dataptr - fitsdat) / 2880) + 1) * 2880;
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen6: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        
        if ((x_ptr = strstr(dataptr, xtension)) == NULL) goto skip3;
/*        printf("dataptr3-fitsdat: %12d, filelen: %12d\n", (int)(x_ptr-fitsdat), filelen);
*/           
        if ((nax1 = strstr(x_ptr, "NAXIS1")) == NULL) {printf("null-pointer 09\n"); exit(0);}
        *(nax1+31) = 0;
        tab_wdth3 = atoi(nax1+16);
/*        printf("Table width3: %5d\n", tab_wdth3);
*/           
        if ((nax2 = strstr(nax1+32, "NAXIS2")) == NULL) {printf("null-pointer 10\n"); exit(0);}
        *(nax2+31) = 0;
        tab_dpth3 = atoi(nax2+16);
/*        printf("Table depth3: %10d\n", tab_dpth3);
*/           
        nax2 += 32;
        if ((end_ptr = strstr(nax2, "END    ")) == NULL) {printf("null-pointer 11\n"); exit(0);}
        if ((end_ptr - x_ptr) % 80 != 0) {
           printf("END -marker found out of place\n");
           exit(0);
        }
           
        if (((end_ptr - fitsdat) % 2880) == 0) dataptr = end_ptr;   
        else dataptr = x_ptr + ((int)((end_ptr - x_ptr) / 2880) + 1) * 2880;
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen7: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 

        for (i=0; i<tab_dpth3; i++) {
           fits_sh    = *dataptr * 256 + *(dataptr+1);
           SSC_3[i]   = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+4) * 256 + *(dataptr+5);
           lobt2_2[i] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+6) * 256 + *(dataptr+7);
           lobt1_2[i] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+8) * 256 + *(dataptr+9);
           lobt0_2[i] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);              
             
           fits_sh    = *(dataptr+10) * 256 + *(dataptr+11);
           pos_gr[i]  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           grey_3[i]  = *(dataptr+12);

           delT_gr[i] = *(dataptr+13);

           if (i < 0) printf("ssc: %5d, lobt2/1/0: %5d %5d %5d, position: %3d, grey: %2d, deltaT: %4d\n",
              (int)(SSC_3[i]), (int)(lobt2_2[i]), (int)(lobt1_2[i]), (int)(lobt0_2[i]), 
              (int)(pos_gr[i]), (int)(grey_3[i]), (int)(delT_gr[i]));

           dataptr += tab_wdth3;
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen8: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        }
        
        startT = (double)(lobt2_1[0]) * 65536.0 + (double)(lobt1_1[0]) + (double)(lobt0_1[0]) / 65536.0;
           
skip3:           
        it3 = 0;
        ssc_old = -10;
        dataptr = listptr;
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen9: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        ssc_grey = 999999999;
        map_open = 0;
        e_rate = 25.0;
           
        for (it1=0; it1<tab_dpth1; it1++) {

           grey = grey_1[it1];
           
           ssc = SSC_1[it1];
           if (ssc_grey < ssc) it3++;
              
           if (it3 < tab_dpth3) {
              ssc_grey = SSC_3[it3];
           } else { 
              ssc_grey = 999999999;
           }

           startT = (double)(lobt2_1[it1])*65536.0 + (double)(lobt1_1[it1]) + (double)(lobt0_1[it1])/65536.0;
           it2=0;

new_file:
   	   if (map_open == 0) {
   	       acc_curT = 0.0;
               sprintf(b_file, "%03d_%04d_%03d_shd%1d.bin", orbit, pid, pidv, jemx+1);
               map_open = 1;

               for (k=0; k<4; k++) 
                  for (i=0; i<256; i++) 
                     for (j=0; j<256; j++) 
                        shdgr[k][j][i] = 0; 

           }
/*           printf("it1: %4d,  depth: %4d,  tab_wdth2: %6d\n", it1, tab_dpth1, tab_wdth2);
*/           
   	   runT = startT;
           for (; it2<n_evts[it1]; it2++) {
              
              if (ssc_grey == ssc) {
                 if (it2 >= pos_gr[it3]) grey = grey_3[it3];
              }
              
              if ((dataptr - fitsdat) > filelen) {
                 printf("Data error!  dataptr-fitsdat: %12d, filelen10: %12d  ... exiting!\n", 
                    (int)(dataptr-fitsdat), filelen);
                    return;
              } 
              x = (int)(*(dataptr));
              y = (int)(*(dataptr+1));
              pha = (int)(*(dataptr+2));
              fits_sh = (short)((int)(*(dataptr+3)) * 256 + (int)(*(dataptr+4)));
              dti = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);
              
              livetime += (double)(dti) / 8192.0 * (grey+1) / 32.0 *
                 (1.0 - 0.109 * (0.9634 - grey * 0.00293) / 0.87 - 0.000188 * e_rate);

              dataptr += tab_wdth2;
              if ((dataptr - fitsdat) > filelen) {
                 printf("Data error!  dataptr-fitsdat: %12d, filelen11: %12d  ... exiting!\n", 
                    (int)(dataptr-fitsdat), filelen);
                 return;
              } 
                               
              if (pha < 2) ppha = 0;
              else ppha = sp[pha] - (jj % dsp[pha]);
              
              pha = ppha / 4;
              if (pha > 254) pha = 255;
              if (dti < 256) waittim[dti]++; else waittim[255]++;
              
              acc_curT += (double)(dti) / 8192.0;
              runT += (double)(dti) / 8192.0;

              pickevent = 0;
              /* correct pulseheight for detector nonuniformity */
              ccpha = pha_xy(x, y, pha, jemx, iname2, 0);
              iipha = (int)(ccpha / 4.0);
              iipha = ccpha;

              if ((iipha >= sel0) && (iipha < sel1)) {     /* <4 keV */ 
                 sev0++;
                 shdgr[0][x][y]++;
              }

              if ((iipha >= sel1) && (iipha < sel2)) {     /* 4 to 8 keV */ 
                 sev1++;
                 shdgr[1][x][y]++;
              }

              if ((iipha >= sel2) && (iipha < sel3)) {     /* 8 to 14 keV */ 
                               sev2++;
                 shdgr[2][x][y]++;
              }

              if ((iipha >= sel3) && (iipha < sel4)) {     /* 14 to 35 keV */ 
                 sev3++;
                 shdgr[3][x][y]++;
              }

              if ((acc_curT > acTlim) && (pidv+1 < divid)) {
   		 fim = fopen(b_file, "wb");
                 if (fim == NULL) {
                    printf("Could not open %s\n", b_file);
                    return;
                 }

                 fwrite(shdgr, 2, 4*65536, fim);
   
                 aux_tot[0] = sev0;
                 aux_tot[1] = sev1;
                 aux_tot[2] = sev2;
                 aux_tot[3] = sev3;

              printf("%s input bytes: %9d, shdg cnts: %7.0lf %7.0lf %7.0lf %7.0lf, time: %6.1lf (live): %6.1lf s\n", 
                    b_file, filelen, aux_tot[0], aux_tot[1], aux_tot[2], aux_tot[3], acc_curT, livetime);
                        
                 aux_tot[4] = orbit * 65536.0 + pid;
                 aux_tot[23] = 376;
                 aux_tot[23] = 377;
                 aux_tot[27] = livetime;
                 livetime = 0.0;
                 aux_tot[28] = pidv;
           
                 fwrite(aux_tot, 8, 30, fim);
                 fclose(fim);
                 sev0 = sev1 = sev2 = sev3 = 0;
                 pidv++;
                 map_open = 0;
                 goto new_file;
              }
           }
           if ((runT - startT) > 0.0) e_rate = n_evts[it1] / (runT - startT);
           else e_rate = 25.0;
           
        }
end:

        fim = fopen(b_file, "wb");
        if (fim == NULL) {
           printf("Could not open %s\n", b_file);
           return;
        }
        
        fwrite(shdgr, 2, 4*65536, fim);
   
        aux_tot[0] = sev0;
        aux_tot[1] = sev1;
        aux_tot[2] = sev2;
        aux_tot[3] = sev3;

        printf("%s input bytes: %9d, shdg cnts: %7.0lf %7.0lf %7.0lf %7.0lf, time: %6.1lf (live): %6.1lf s\n", 
           b_file, filelen, aux_tot[0], aux_tot[1], aux_tot[2], aux_tot[3], acc_curT, livetime);
                        
        aux_tot[4] = orbit * 65536.0 + pid;
        aux_tot[23] = 376;
        aux_tot[23] = 377;
        aux_tot[27] = livetime;
        livetime = 0.0;
        aux_tot[28] = pidv;
           
        fwrite(aux_tot, 8, 30, fim);
        fclose(fim);
      
}

/* ******************************************************************************** */

float pha_xy(int x, int y, int pha, int jemx, char iname2[], int print)
{
   FILE *xyfil, *afil, *gnufil;
   char filenam[200], iline[200];
   static int data[2] = {0, 0};
   static float xy_cor[2][256][256], acor[2][256];
   static int evnum[2], spectrum[3][256];
   float cpha, r;
   int i, j, k, n, use;
   
   use = 1; 
   if (print) goto skriv;

   r = sqrt((double)((x-127)*(x-127) + (y-127)*(y-127)));
   if (r > 90.0) use = 0;
   
   if (data[jemx] == 0) {
      if (jemx == 0) sprintf(filenam, "%s/reference/fs_gain_cor.dat", sweethome);
      else sprintf(filenam, "%s/reference/fm2_gain_cor.dat", sweethome);
      xyfil = fopen(filenam, "rt");
      if (xyfil == NULL) {
         printf("Could not open %s\n", filenam);
         exit(0);
      }
      
      for (j=0; j<256; j++) {
         for (k=0; k<42; k++) {
            i = k * 6;
            fgets(iline, 128, xyfil);
            sscanf(iline, "%f %f %f %f %f %f", &xy_cor[jemx][i][j], &xy_cor[jemx][i+1][j],
               &xy_cor[jemx][i+2][j], &xy_cor[jemx][i+3][j], &xy_cor[jemx][i+4][j], &xy_cor[jemx][i+5][j]);
         }
         fgets(iline, 128, xyfil);
         i = 252;
         sscanf(iline, "%f %f %f %f", 
            &xy_cor[jemx][i][j], &xy_cor[jemx][i+1][j], &xy_cor[jemx][i+2][j], &xy_cor[jemx][i+3][j]);
      }
      fclose(xyfil);
      
      sprintf(filenam, "%s/reference/a_factors%1d.txt", sweethome, jemx+1);
      afil = fopen(filenam, "rt");
      if (afil == NULL) {
         printf("Could not open %s\n", filenam);
         exit(0);
      }
      
      for (j=0; j<256; j++) {
         fgets(iline, 128, afil);
         sscanf(iline, "%d %f", &k, &acor[jemx][j]);
      }
      fclose(afil);

      data[jemx] = 1;
   }
   
   cpha = (float)(pha) / xy_cor[jemx][x][y];
   
   if ((cpha >= 0.0) && (cpha < 1024.0)) {
      if (use) spectrum[1][(int)(cpha/4.0)]++;
   }
   if (acor[jemx][x] > 0.0) cpha /= 1.0 + (acor[jemx][x] - 1.0);
   
   if ((cpha >= 0.0) && (cpha < 1024.0)) {
      if (use) spectrum[2][(int)(cpha/4.0)]++;
   }
   
   return(cpha);
   
skriv:
   if (evnum[jemx] > 0) {
      spectrum[0][0] = spectrum[1][0] = spectrum[2][0] = 0;
      printf("\nJEM-X%1d counts: %6d\n", jemx+1, evnum[jemx]);
      sprintf(filenam, "%s_spec.gnu", iname2);
      gnufil = fopen(filenam, "wt");
   
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6d\n", j * 30.0 / 160.0, spectrum[0][j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6d\n", j * 30.0 / 160.0, spectrum[1][j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6d\n", j * 30.0 / 160.0, spectrum[2][j]);
      fprintf(gnufil, "\n\n");
      fclose(gnufil);
      evnum[jemx] = 0;
   }
   
   return(1.0);
}

/* ******************************************************************************** */

